#!/usr/bin/env python2

import sys, os, getopt, string

def writeXMLSpec(packageInfo):
    """Write package specification in xml format"""

    try:
        file = open("%s.xml" % packageInfo['filename'], 'w')
    except IOError:
        sys.stderr.write("Error writing file %s.xml\n" %  packageInfo['filename'])

    file.write('<?xml version="1.0" encoding="utf-8"?>\n')
    file.write("<packagelist>\n<package>\n")
    file.write("<name>%s</name>\n" % packageInfo['name'])    
    file.write("<version>%s</version>\n" % packageInfo['version'])
    for x in packageInfo['author']:
        file.write('<author name="%s" email="%s" />\n' % (x[0], x[1]))
    file.write("<license>%s</license>\n" % packageInfo['license'])
    file.write("<description>%s</description>\n" % packageInfo['description'])
    file.write("<url>%s</url>\n" % packageInfo['url'])
    file.write("<filename>%s</filename>\n" % packageInfo['filename'])
    file.write("<dependencies>\n")
    dependencies = packageInfo['dependencies']
    for x in packageInfo['dependencies']:
        file.write('<dependency type="%s">\n' % x['type'])
        file.write('<name>%s</name>\n' % x['name'])
        file.write('<version type="%s">%s</version>' % (x['version-type'],
                                                        x['version']))
        file.write('</dependency>\n')
    file.write("</dependencies>\n")
    file.write("</package>\n")
    file.write("</packagelist>\n")
    file.close()

def writePEPSpec(packageInfo):
    """Write package specification in PEP 241 format"""
    try:
        file = open("%s.spec" % packageInfo['filename'], 'w')
    except IOError:
        sys.stderr.write("Error writing file %s.spec\n" %  packageInfo['filename'])

    file.write("Name: %s\n" % packageInfo['name'])    
    file.write("Version: %s\n" % packageInfo['version'])
    for x in packageInfo['author']:
        file.write('Author-email: %s <%s>\n' % (x[0], x[1]))
    file.write("License: %s\n" % packageInfo['license'])
    file.write("Description: %s\n" % packageInfo['description'])
    file.write("Url: %s\n" % packageInfo['url'])
    file.write("Filename: %s\n" % packageInfo['filename'])
    for x in packageInfo['dependencies']:
        file.write('Dependency: %s' % x['name'])
        if x['version-type'] == 'MINIMUM': 
            file.write(' >= %s' % x['version'])
        else:
            file.write(' < %s' % x['version'])

    file.close()

def getDependencyInfo():
    """Obtain dependency information for package and return it as a list"""
    input = ''
    dependencies = []

    # information about dependency
    name = ''
    type = ''
    versionType = ''
    version = ''

    dependencyTypes = "(1) Require\n(2) Suggest\n(3) External\n(4) Exit\n"
    versionTypes = "(1) Minimum\n(2) Maximum\n"

    while 1:
        
        sys.stdout.write(dependencyTypes)
        sys.stdout.write("Enter dependency type: ")
        
        input = string.strip(sys.stdin.readline())
        if input == '1':
            type = 'REQUIRE'
        elif input == '2':
            type = 'SUGGEST'
        elif input == '3':
            type = 'EXTERNAL'
        else:
            break

        input = ''
        while len(input) == 0:
            sys.stdout.write("Enter dependency name: ")
            input = string.strip(sys.stdin.readline())

        name = input

        input = ''
        while len(input) == 0:
            sys.stdout.write(versionTypes)
            sys.stdout.write("Enter version modifier: ")
            input = string.strip(sys.stdin.readline())
            if input == '1':
                input = 'MINIMUM'
            elif input == '2':
                input = 'MAXIMUM'
            else:
                input = ''
        versionType = input

        input = ''
        while len(input) == 0:
            sys.stdout.write("Enter dependency version: ")
            input = string.strip(sys.stdin.readline())
        verson = input
        dependencies.append({'type': type,
                             'version': version,
                             'name': name,
                                 'version-type' : versionType})
    return dependencies
            
                
            
            
        

def getAuthorInfo():
    """Obtain author information for package and return it as a list"""

    quit = 0
    input = ''
    authors = []
    name = ''
    email = ''
    while not quit:
        sys.stdout.write("Enter author name (a return here goes to next option): ")
        input = string.strip(sys.stdin.readline())
        if len(input) == 0:
            quit = 1
        else:
            name = input
            input = ''
            while len(input) == 0:
                sys.stdout.write("Enter author's email: ")
                input = string.strip(sys.stdin.readline())
            email = input
            authors.append((name, email))
    return authors

def getDescription():
    """Obtain description for package and return it"""

    quit = 0
    input = ''
    description = ''
    sys.stdout.write("Enter description (press return to go to next option): ")
    while not quit:
        input = string.strip(sys.stdin.readline())
        if len(input) == 0:
            quit = 1
        else:
            description += input
    return description
        
    
xmlOutput = 1
packageInfo = {}
input = ''

try:
    optionList, arguments = getopt.getopt(sys.argv[1:], 'p')
except getopt.error, error:
    sys.stderr.write("Unrecognized option %s\n" % error)
    help()
    sys.exit(1)


for (option, text) in optionList:
    if option == '-p':
        xmlOutput = 0

while len(input) == 0:
    sys.stdout.write("Enter package name: ")
    input = string.strip(sys.stdin.readline())

packageInfo['name'] = input

input = ''

while len(input) == 0:
    sys.stdout.write("Enter package version: ")
    input = string.strip(sys.stdin.readline())

packageInfo['version'] = input

packageInfo['author'] = getAuthorInfo()

packageInfo['description'] = getDescription()

input = ''

while len(input) == 0:
    sys.stdout.write("Enter package license: ")
    input = string.strip(sys.stdin.readline())

packageInfo['license'] = input

input = ''
while len(input) == 0:
    sys.stdout.write("Enter package url: ")
    input = string.strip(sys.stdin.readline())

packageInfo['url'] = input

sys.stdout.write("Enter package filename [%s]: " % packageInfo['name'])
input = string.strip(sys.stdin.readline())

if len(input) == 0:
    packageInfo['filename'] = packageInfo['name']
else:
    packageInfo['filename'] = input

packageInfo['dependencies'] = getDependencyInfo()

if xmlOutput:
    writeXMLSpec(packageInfo)
else:
    writePEPSpec(packageInfo)
    
